==> paragraphs_gantt-1.0.1.tar.gz: paragraphs_gantt/src/Controller/ParagraphsGanttController.php <==
<?php

namespace Drupal\paragraphs_gantt\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Entity\EntityInterface;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs\ParagraphsTypeInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class definition for ComponentFormController.
 */
class ParagraphsGanttController extends ControllerBase {

  /**
   * Query POST.
   *
   * @var array
   */
  private $post = [];

  /**
   * Query GET.
   *
   * @var array
   */
  private $get = [];

  /**
   * {@inheritDoc}
   */
  public function __construct() {
    $request = \Drupal::request();
    $this->post = $request->request->all();
    $this->get = $request->query->all();
  }

  /**
   * Add new / Edit / Delete Paragraphs.
   */
  public function ajax(ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {

    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);

    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    $action = !empty($this->post["!nativeeditor_status"]) ? $this->post["!nativeeditor_status"] : FALSE;
    $output = [];
    if (!$action) {
      return new JsonResponse($output);
    }
    if ($this->get['gantt_mode'] == 'tasks') {
      $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
    }
    if ($this->get['gantt_mode'] == 'links') {
      $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
    }
    return new JsonResponse($output);
  }

  /**
   * Import file MPP.
   */
  public function import(ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {
    $action = 'inserted';
    $output = ['action' => $action];
    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);
    $mapping = [];
    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    if (!empty($this->post["data"])) {
      $rawData = $this->post["data"];
      $rawLinks = $this->post["links"];
      $this->post = [];
      foreach ($rawData as $data) {
        $ganttData[$data['id']] =
        $this->post = [
          'text' => $data['text'],
          'duration' => $data['duration'],
          'progress' => $data['progress'],
          'open' => $data['open'],
          'parent' => $data['parent'],
          'type' => $data['type'],
          'start_date' => $data['$raw']["Start"],
          'end_date' => $data['$raw']["Finish"],
        ];
        if (!empty($mapping[$data['parent']])) {
          $this->post['parent'] = $mapping[$data['parent']];
        }
        $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
        $mapping[$data['id']] = $output['tid'];
      }

      foreach ($rawLinks as $data) {
        $this->post = [
          'source' => $mapping[$data['source']],
          'target' => $mapping[$data['target']],
          'type' => $data['type'],
        ];
        $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
      }
    }

    return new JsonResponse($output);
  }

  /**
   * Convert date return with format Y-m-d.
   */
  protected function formatDate($date, $type = 'date', $format = 'Y-m-d H:i:s') {
    $drupalShortFormat = DateFormat::load('short')->getPattern();
    if ($type == 'date') {
      $format = current(explode(' ', $format));
      $drupalShortFormat = current(explode(' ', $drupalShortFormat));
    }
    $convertDateTime = \DateTime::createFromFormat($drupalShortFormat, $date);
    if (!$convertDateTime) {
      $date = str_replace('/', '-', $date);
      $timestamp = strtotime($date);
      return date($format, $timestamp);
    }
    return $convertDateTime->format($format);
  }

  /**
   * Process Task.
   */
  private function actionTask($action, $entity, $paragraph_type, $entity_field, $field_settings) {
    $output = ['action' => $action];
    if (!empty($this->post['isImport'])) {
      return new JsonResponse($output);
    }
    if ($action == 'inserted') {
      $paragraph = Paragraph::create(['type' => $paragraph_type->id()]);
    }
    else {
      $paragraph = Paragraph::load($this->post["id"]);
    }

    if ($action == 'deleted') {
      $getField = $entity->get($entity_field);
      $array_of_referenced_items = $getField->getValue();
      $index_to_remove = array_search($this->post['id'], array_column($array_of_referenced_items, 'target_id'));
      $getField->removeItem($index_to_remove);
      $entity->save();
      $paragraph->delete();
      return new JsonResponse($output);
    }

    $fieldDefinitions = $paragraph->getFieldDefinitions();
    $fieldType = $dateType = 'date';
    if (!empty($fieldDefinitions[$field_settings['start_date']])) {
      $dateSettings = $fieldDefinitions[$field_settings['start_date']]->getSettings();
      $dateType = $dateSettings["datetime_type"];
      $fieldType = $fieldDefinitions[$field_settings['start_date']]->getType();
    }
    $dateFormat = $fieldType == 'daterange' ? 'Y-m-d\TH:i:s' : 'Y-m-d H:i:s';
    $start_date = $this->formatDate($this->post['start_date'], $dateType, $dateFormat);
    $end_date = $this->formatDate($this->post['end_date'], $dateType, $dateFormat);
    if ($fieldType == 'daterange') {
      $start_date = [
        'value' => $start_date,
        'end_value' => $end_date,
      ];
    }
    $paragraph->set($field_settings['text'], $this->post['text']);
    $paragraph->set($field_settings['duration'], $this->post['duration']);
    $paragraph->set($field_settings['start_date'], $start_date);
    if (!empty($this->post['progress'])) {
      $paragraph->set($field_settings['progress'], $this->post['progress']);
    }
    $parent = !empty($this->post['parent']) ? $this->post['parent'] : 0;
    $paragraph->set($field_settings['parent'], $parent);
    $open = !empty($this->post['open']) ? $this->post['open'] : 0;
    $paragraph->set($field_settings['open'], $open);
    if (!empty($field_settings["owner"])) {
      $owner = [];
      if (is_array($this->post['owner'])) {
        foreach ($this->post['owner'] as $owner_id) {
          $owner[] = ['target_id' => $owner_id];
        }
      }
      else {
        $owner = [['target_id' => $this->post['owner']]];
      }
      $paragraph->set($field_settings['owner'], $owner);
    }
    if ($action == 'inserted') {
      $paragraph->isNew();
    }
    $paragraph->save();

    if ($action == 'inserted') {
      $output['tid'] = $paragraph->id();
      $fieldValue = $entity->get($entity_field)->getValue();
      $fieldValue[] = [
        'target_id' => $paragraph->id(),
        'target_revision_id' => $paragraph->getRevisionId(),
      ];
      $entity->set($entity_field, $fieldValue);
      $entity->save();
    }

    return $output;
  }

  /**
   * {@inheritDoc}
   */
  private function actionLinks(string $action, EntityInterface $entity, ParagraphsTypeInterface $paragraph_type, string $entity_field, array $field_settings) {
    $output = ['action' => $action];
    $source = !empty($this->post['source']) ? $this->post['source'] : FALSE;
    $target = !empty($this->post['target']) ? $this->post['target'] : FALSE;
    $type = !empty($this->post['type']) ? $this->post['type'] : FALSE;
    if ($action == 'deleted') {
      [$source, $target] = explode('-', $this->post['id']);
    }
    $paragraph = Paragraph::load($source);
    $linkValues = $paragraph->get($field_settings['links'])->getValue();

    if ($action == 'inserted' || !empty($this->post['isImport'])) {
      $linkValues[] = ['first' => $target, 'second' => $type];
    }
    if ($action == 'deleted') {
      foreach ($linkValues as $delta => $linkValue) {
        if ($linkValue['first'] == $target) {
          unset($linkValues[$delta]);
        }
      }
    }
    $paragraph->set($field_settings['links'], $linkValues);

    $paragraph->save();

    return new JsonResponse($output);
  }

}

==> paragraphs_gantt-1.0.3.tar.gz: paragraphs_gantt/src/Controller/ParagraphsGanttController.php <==
<?php

namespace Drupal\paragraphs_gantt\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs\ParagraphsTypeInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class definition for ComponentFormController.
 */
class ParagraphsGanttController extends ControllerBase {

  /**
   * Query POST.
   *
   * @var mixed
   */
  private $post;

  /**
   * Add new / Edit / Delete Paragraphs.
   */
  public function ajax(Request $request, ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {

    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);

    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    $action = $request->request->get("!nativeeditor_status") ?? FALSE;
    $output = [];
    if (!$action) {
      return new JsonResponse($output);
    }
    $this->post = $request->request->all();
    $mode = $request->query->get('gantt_mode');
    if ($mode == 'tasks') {
      $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
    }
    if ($mode == 'links') {
      $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
    }
    return new JsonResponse($output);
  }

  /**
   * Import file MPP.
   */
  public function import(ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {
    $action = 'inserted';
    $output = ['action' => $action];
    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);
    $mapping = [];
    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    if (!empty($request->request->get("data"))) {
      $rawData = $this->request->get("data");
      $rawLinks = $this->request->get("links");
      $this->post = [];
      foreach ($rawData as $data) {
        $ganttData[$data['id']] =
        $this->post = [
          'text' => $data['text'],
          'duration' => $data['duration'],
          'progress' => $data['progress'],
          'open' => $data['open'],
          'parent' => $data['parent'],
          'type' => $data['type'],
          'start_date' => $data['$raw']["Start"],
          'end_date' => $data['$raw']["Finish"],
        ];
        if (!empty($mapping[$data['parent']])) {
          $this->post['parent'] = $mapping[$data['parent']];
        }
        $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
        $mapping[$data['id']] = $output['tid'];
      }

      foreach ($rawLinks as $data) {
        $this->post = [
          'source' => $mapping[$data['source']],
          'target' => $mapping[$data['target']],
          'type' => $data['type'],
        ];
        $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
      }
    }

    return new JsonResponse($output);
  }

  /**
   * Convert date return with format Y-m-d.
   */
  protected function formatDate($date, $type = 'date', $format = 'Y-m-d H:i:s') {
    $dateFormatStorage = $this->entityTypeManager->getStorage('date_format');
    $drupalShortFormat = $dateFormatStorage->load('short')->getPattern();
    if ($type == 'date') {
      $format = current(explode(' ', $format));
      $drupalShortFormat = current(explode(' ', $drupalShortFormat));
    }
    $convertDateTime = \DateTime::createFromFormat($drupalShortFormat, $date);
    if (!$convertDateTime) {
      $date = str_replace('/', '-', $date);
      $timestamp = strtotime($date);
      return date($format, $timestamp);
    }
    return $convertDateTime->format($format);
  }

  /**
   * Process Task.
   */
  private function actionTask($action, $entity, $paragraph_type, $entity_field, $field_settings) {
    $output = ['action' => $action];
    if (!empty($this->post['isImport'])) {
      return new JsonResponse($output);
    }
    if ($action == 'inserted') {
      $paragraph = Paragraph::create(['type' => $paragraph_type->id()]);
    }
    else {
      $paragraph = Paragraph::load($this->post["id"]);
    }

    if ($action == 'deleted') {
      $getField = $entity->get($entity_field);
      $array_of_referenced_items = $getField->getValue();
      $index_to_remove = array_search($this->post['id'], array_column($array_of_referenced_items, 'target_id'));
      $getField->removeItem($index_to_remove);
      $entity->save();
      $paragraph->delete();
      return new JsonResponse($output);
    }

    $fieldDefinitions = $paragraph->getFieldDefinitions();
    $fieldType = $dateType = 'date';
    if (!empty($fieldDefinitions[$field_settings['start_date']])) {
      $dateSettings = $fieldDefinitions[$field_settings['start_date']]->getSettings();
      $dateType = $dateSettings["datetime_type"];
      $fieldType = $fieldDefinitions[$field_settings['start_date']]->getType();
    }
    $dateFormat = $fieldType == 'daterange' ? 'Y-m-d\TH:i:s' : 'Y-m-d H:i:s';
    $start_date = $this->formatDate($this->post['start_date'], $dateType, $dateFormat);
    $end_date = $this->formatDate($this->post['end_date'], $dateType, $dateFormat);
    if (!empty($this->post['planned_start'])) {
      $planned_start = $this->formatDate($this->post['planned_start'], $dateType, $dateFormat);
      $planned_end = $this->formatDate($this->post['planned_end'], $dateType, $dateFormat);
      $planned_date = [
        'value' => $planned_start,
        'end_value' => $planned_end,
      ];
      $paragraph->set($field_settings['planned_date'], $planned_date);
      if (!empty($field_settings['planned_duration'])) {
        $planned_duration = round(abs(strtotime($planned_start) - strtotime($planned_end)) / 86400);
        $paragraph->set($field_settings['planned_duration'], $planned_duration);
      }
    }
    if ($fieldType == 'daterange') {
      $start_date = [
        'value' => $start_date,
        'end_value' => $end_date,
      ];
    }
    $paragraph->set($field_settings['text'], $this->post['text']);
    $paragraph->set($field_settings['duration'], $this->post['duration']);
    $paragraph->set($field_settings['start_date'], $start_date);
    if (!empty($this->post['progress'])) {
      $paragraph->set($field_settings['progress'], $this->post['progress']);
    }
    $parent = !empty($this->post['parent']) ? $this->post['parent'] : 0;
    $paragraph->set($field_settings['parent'], $parent);
    $open = !empty($this->post['open']) ? $this->post['open'] : 0;
    $paragraph->set($field_settings['open'], $open);
    if (!empty($field_settings["owner"])) {
      $owner = [];
      if (is_array($this->post['owner'])) {
        foreach ($this->post['owner'] as $owner_id) {
          $owner[] = ['target_id' => $owner_id];
        }
      }
      else {
        $owner = [['target_id' => $this->post['owner']]];
      }
      $paragraph->set($field_settings['owner'], $owner);
    }
    if ($action == 'inserted') {
      $paragraph->isNew();
    }
    $paragraph->save();

    if ($action == 'inserted') {
      $output['tid'] = $paragraph->id();
      $fieldValue = $entity->get($entity_field)->getValue();
      $fieldValue[] = [
        'target_id' => $paragraph->id(),
        'target_revision_id' => $paragraph->getRevisionId(),
      ];
      $entity->set($entity_field, $fieldValue);
    }
    $entity->save();

    return $output;
  }

  /**
   * {@inheritDoc}
   */
  private function actionLinks(string $action, EntityInterface $entity, ParagraphsTypeInterface $paragraph_type, string $entity_field, array $field_settings) {
    $output = ['action' => $action];
    $source = !empty($this->post['source']) ? $this->post['source'] : FALSE;
    $target = !empty($this->post['target']) ? $this->post['target'] : FALSE;
    $type = !empty($this->post['type']) ? $this->post['type'] : FALSE;
    if ($action == 'deleted') {
      [$source, $target] = explode('-', $this->post['id']);
    }
    $paragraph = Paragraph::load($source);
    $linkValues = $paragraph->get($field_settings['links'])->getValue();

    if ($action == 'inserted' || !empty($this->post['isImport'])) {
      $linkValues[] = ['first' => $target, 'second' => $type];
    }
    if ($action == 'deleted') {
      foreach ($linkValues as $delta => $linkValue) {
        if ($linkValue['first'] == $target) {
          unset($linkValues[$delta]);
        }
      }
    }
    $paragraph->set($field_settings['links'], $linkValues);

    $paragraph->save();

    return new JsonResponse($output);
  }

}
